package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.ArrayDataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;

public class RelatorioBPe extends Relatorio {

	public RelatorioBPe(Map<String, Object> parametros, Connection conexao) throws Exception {

		super(parametros, conexao);

		this.setCustomDataSource(new ArrayDataSource(this) {

			public void initDados() throws Exception {
				Connection conexao = this.relatorio.getConexao();
				Map<String, Object> parametros = this.relatorio.getParametros();

				String sql = getSql(parametros);
				
				Date dtInicio = (Date) parametros.get("DATA_INICIO");
				Date dtFim = (Date) parametros.get("DATA_FIM");

				PreparedStatement ps = conexao.prepareStatement(sql.toString());
				ps.setString(1, DateUtil.getStringDate(dtInicio, "dd/MM/yyyy") + " 00:00:00");
				ps.setString(2, DateUtil.getStringDate(dtFim, "dd/MM/yyyy") + " 23:59:59");
				
				ResultSet rset = ps.executeQuery();

				while (rset.next()) {
					Map<String, Object> dataResult = new HashMap<String, Object>();

					dataResult.put("dtvenda", rset.getString("dtvenda"));
					dataResult.put("hrvenda", rset.getString("hrvenda"));
					dataResult.put("origem", rset.getString("origem"));
					dataResult.put("destino", rset.getString("destino"));
					dataResult.put("dtviagem", rset.getString("dtviagem"));
					dataResult.put("vlbpe", rset.getString("vlbpe"));
					dataResult.put("chbpe", rset.getString("chbpe"));
					dataResult.put("num_bpe", rset.getString("num_bpe"));
					dataResult.put("numserie_bpe", rset.getString("numserie_bpe"));
					dataResult.put("status", rset.getString("status"));
					dataResult.put("obs", rset.getString("obs"));
					dataResult.put("qrcode", rset.getString("qrcode"));
					dataResult.put("errocontingencia", rset.getString("errocontingencia"));
					
					this.dados.add(dataResult);
				}

				this.resultSet = rset;
			}
		});
	}

	@Override
	protected void processaParametros() throws Exception {
	}

	private String getSql(Map<String, Object> parametros) {

		StringBuilder sql = new StringBuilder();

		String estados = (String) parametros.get("ESTADOS_ID");
		Integer empresaId = (Integer) parametros.get("EMPRESA_ID");
		String status = (String) parametros.get("STATUS");

		sql.append("SELECT");
		sql.append("  TO_CHAR(COALESCE(bpe.DT_VENDA,bol.FECHORVENTA),'dd/mm/yyyy') as dtvenda, ");
		sql.append("  TO_CHAR(COALESCE(bpe.DT_VENDA,bol.FECHORVENTA),'HH24:MI') as hrvenda, ");
		sql.append("  ori.DESCPARADA as origem,  ");
		sql.append("  dest.DESCPARADA as destino,");
		sql.append("  TO_CHAR(bol.FECHORVIAJE,'dd/mm/yyyy HH24:MI') as dtviagem, ");
		sql.append("  TO_CHAR(COALESCE(bol.PRECIOPAGADO,0) + coalesce(bol.IMPORTETAXAEMBARQUE,0) + coalesce(bol.IMPORTESEGURO,0) + coalesce(bol.IMPORTEPEDAGIO,0) + coalesce(bol.IMPORTEOUTROS,0))  as vlbpe, ");
		sql.append("  bpe.CHBPE,  ");
		sql.append("  bol.NUM_BPE as num_bpe, ");
		sql.append("  COALESCE(bol.NUMSERIE_BPE, '1') as numserie_bpe, ");
		sql.append("  CASE bpe.CODSTAT ");
		sql.append("  WHEN '100' THEN (CASE WHEN bpe.TIPOSUBSTITUICAO IS NULL THEN 'Autorizado'  ELSE 'Substitudo' END ) ");
		sql.append("  WHEN '102' THEN (CASE WHEN bpe.TIPOSUBSTITUICAO IS NOT NULL THEN 'Substitudo' END) ");
		sql.append("  WHEN '135' THEN (CASE bpe.TIPOEVENTO WHEN  '110111' THEN 'Cancelado' WHEN '110115' THEN 'No embarcado' ELSE NULL END) ");
		sql.append("  WHEN '150' THEN 'Aut.Conting.' ");
		sql.append("  ELSE 'Rejeitado' END as status, ");
		sql.append("  CASE WHEN bpe.CODSTAT not in ('100','135') THEN bpe.motivo  ELSE NULL END as obs, ");
		sql.append("  bpe.QRCODE, bpe.errocontingencia ");

		sql.append(" FROM BPE bpe ");
		sql.append(" LEFT JOIN BOLETO bol ON bol.BOLETO_ID = bpe.BOLETO_ID and bpe.activo = 1  ");
		sql.append(" LEFT JOIN estado e ON e.CODIBGE = bpe.UF  ");
		sql.append(" LEFT JOIN marca ma ON bol.MARCA_ID = ma.MARCA_ID  ");
		sql.append(" LEFT JOIN empresa ep on ma.EMPRESA_ID = ep.EMPRESA_ID ");
		sql.append(" LEFT JOIN PARADA ori on bol.ORIGEN_ID = ori.PARADA_ID ");
		sql.append(" LEFT JOIN PARADA dest on bol.DESTINO_ID = dest.PARADA_ID ");

		sql.append(" WHERE e.ACTIVO = 1 ");

		if (empresaId != null) {
			sql.append(" AND ep.EMPRESA_ID =  " + empresaId + " ");
		}
		if (estados != null) {
			sql.append(" AND e.ESTADO_ID IN ( " + estados + " )");
		}

		if (status != null && status.equals("A")) {
			sql.append(" AND  bpe.CODSTAT = '100' AND bpe.TIPOSUBSTITUICAO IS NULL AND bol.ACTIVO = 1 ");
		} else if (status != null && status.equals("C")) {
			sql.append(" AND  bpe.CODSTAT = '135' AND bpe.TIPOEVENTO = '110111' AND bol.ACTIVO = 1 ");
		} else if (status != null && status.equals("S")) {
			sql.append(" AND bpe.CODSTAT IN ('100', '102') AND bpe.TIPOSUBSTITUICAO IS NOT NULL AND bol.ACTIVO = 1 ");
		} else if (status != null && status.equals("NE")) {
			sql.append(" AND bpe.CODSTAT = '135' AND bpe.TIPOEVENTO = '110115' AND bol.ACTIVO = 1 ");
		} else if (status != null && status.equals("R")) {
			sql.append(" AND bpe.CODSTAT NOT IN ('100','135','102','150') ");
		}else if (status != null && status.equals("AA")) {
			sql.append(" AND  bpe.CODSTAT = '150' AND bpe.TIPOSUBSTITUICAO IS NULL AND bol.ACTIVO = 1 ");
		}else if (status != null && status.equals("RC")) {
			sql.append(" AND  bpe.CODSTAT IN ('-10','-20') AND bol.ACTIVO = 1 ");
		}

		sql.append(" AND COALESCE(bpe.DT_VENDA,bol.FECHORVENTA) >= TO_DATE(?,'DD/MM/YYYY HH24:MI:SS') ");
		sql.append(" AND COALESCE(bpe.DT_VENDA,bol.FECHORVENTA) <= TO_DATE(?,'DD/MM/YYYY HH24:MI:SS') ");

		sql.append(" ORDER BY bol.NUM_BPE, COALESCE(bol.NUMSERIE_BPE, '1')  ASC ");

		return sql.toString();
	}
}