package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioGratuidadeANTTBean;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

public class RelatorioGratuidadeANTT extends Relatorio {

	private List<RelatorioGratuidadeANTTBean> lsDadosRelatorio;
	
	public RelatorioGratuidadeANTT(Map<String, Object> parametros, Connection conexao)  throws Exception {
		super(parametros, conexao);
		
		this.setCustomDataSource(new DataSource(this) {
			
			@Override
			public void initDados() throws Exception {

				Connection conexao = this.relatorio.getConexao();

				Map<String, Object> parametros = this.relatorio.getParametros();

				String fecInicioVenda = null;
				if (parametros.get("fecInicioVenda") != null) {
					fecInicioVenda = parametros.get("fecInicioVenda").toString() + " 00:00:00";
				}
				String fecFinalVenda = null;
				if (parametros.get("fecFinalVenda") != null) {
					fecFinalVenda = parametros.get("fecFinalVenda").toString() + " 23:59:59";
				}
				
				String tipGratuIds = parametros.get("tipGratuIds").toString();
				String linhaIds = parametros.get("linhaIds").toString();
				String empresa = parametros.get("empresa") != null ? parametros.get("empresa").toString() : "";
						
				String sql = getSql(fecInicioVenda, fecFinalVenda, linhaIds, tipGratuIds, empresa);
		
				SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");
				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);

				if (fecInicioVenda != null) {
					stmt.setTimestamp("fecInicioVenda", new java.sql.Timestamp(sdf.parse(fecInicioVenda).getTime()));
				}
				
				if (fecFinalVenda != null) {
					stmt.setTimestamp("fecFinalVenda", new java.sql.Timestamp(sdf.parse(fecFinalVenda).getTime()));
				}
				
				if(empresa != null && !empresa.equals("")){
					stmt.setInt("empresa_id", Integer.parseInt(empresa));
				}
				
				ResultSet rset = null;
		
				rset = stmt.executeQuery();
		
				lsDadosRelatorio = new ArrayList<RelatorioGratuidadeANTTBean>();
				
				while (rset.next()) {
					RelatorioGratuidadeANTTBean gratuidadeBean = new RelatorioGratuidadeANTTBean();
					
					gratuidadeBean.setCategoria(rset.getString("categoria"));

					gratuidadeBean.setDescOrigem(rset.getString("descOrigem"));
					gratuidadeBean.setDescDestino(rset.getString("descDestino"));
					gratuidadeBean.setCodOrigem(rset.getString("codOrigem"));
					gratuidadeBean.setCodDestino(rset.getString("codDestino"));
					gratuidadeBean.setTotIda(rset.getInt("totalIda"));
					gratuidadeBean.setTotVolta(rset.getInt("totalVolta"));


					lsDadosRelatorio.add(gratuidadeBean);
				}

				if (lsDadosRelatorio.size() > 0) {
					setLsDadosRelatorio(lsDadosRelatorio);
				}
			}
		});
	}
	
	public void setLsDadosRelatorio(List<RelatorioGratuidadeANTTBean> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
		this.lsDadosRelatorio = lsDadosRelatorio;
	}

	@Override
	protected void processaParametros() throws Exception {
	}
	
	private String getSql(String fecInicioVenda, String fecFinalVenda, String linha, String tipoGratu, String empresa ) {

		StringBuilder sql = new StringBuilder();

		sql.append(" select  ");
		sql.append("   ca.DESCCATEGORIA AS categoria,  ");
		sql.append("   ori.CVEPARADA AS codOrigem,  ");
		sql.append("   ori.DESCPARADA AS descOrigem,  ");
		sql.append("   des.CVEPARADA AS codDestino,  ");
		sql.append("   des.DESCPARADA AS descDestino,   ");
		sql.append("   sum(case when (R.INDSENTIDOIDA = 1 ) then 1 else 0 end) as totalIda,  ");
		sql.append("   sum(case when (R.INDSENTIDOIDA = 0 ) then 1 else 0 end) as totalVolta ");
		sql.append(" from BOLETO b ");
		sql.append(" JOIN categoria ca ON b.categoria_id = ca.categoria_id ");
		sql.append(" JOIN parada ori ON ori.parada_id = b.origen_id ");
		sql.append(" JOIN parada des ON des.parada_id = b.destino_id ");
		sql.append(" JOIN ruta r ON r.ruta_id = b.ruta_id ");
		sql.append(" WHERE b.fechorventa BETWEEN :fecInicioVenda AND :fecFinalVenda ");
		
		if( tipoGratu != null ) {
			sql.append(" AND b.CATEGORIA_ID in (").append(tipoGratu).append(") ");
		}
		
		if( linha != null && !linha.equals("Todas")) {
			sql.append(" AND r.ruta_id in (").append(linha).append(") ");
		}
		
		if (!empresa.isEmpty()){
			sql.append("AND b.empresacorrida_id = :empresa_id  ");
		}
		
		sql.append(" group by ca.DESCCATEGORIA, ori.CVEPARADA, ori.DESCPARADA, des.CVEPARADA, des.DESCPARADA ");
		sql.append(" ORDER BY descOrigem, descDestino, categoria ");

		return sql.toString();
	}
}