package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.Calendar;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Radio;

import com.rjconsultores.ventaboletos.entidad.ContaMD;
import com.rjconsultores.ventaboletos.service.ContaMDService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextboxDecimal;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextboxSimple;

@Controller("editarContaMDController")
@Scope("prototype")
public class EditarContaMDController extends MyGenericForwardComposer {
	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(EditarContaMDController.class);

	@Autowired
	private ContaMDService contaMDService;

	private ContaMD conta;
	private MyListbox contaList;
	private MyTextboxDecimal txtCodigoContabil;
	private MyTextboxSimple txtNome;
	private Radio radioTipoC;
	private Radio radioTipoD;
	private Radio radioCancelamentoS;
	private Radio radioCancelamentoN;
	private Radio radioStatusA;
	private Radio radioStatusI;
	private Radio radioOrigemIN;
	private Radio radioOrigemEE;
	private Radio radioOrigemFP;
	private MyTextbox txtReferencia;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		conta = (ContaMD) Executions.getCurrent().getArg().get("conta");
		contaList = (MyListbox) Executions.getCurrent().getArg().get("contaList");

		if (conta != null) {
			if (conta.getCodigoContabil() != null) {
				txtCodigoContabil.setValue(conta.getCodigoContabil().toString());
			}
			
			if (conta.getNome() != null) {
				txtNome.setText(conta.getNome());
			}

			if (conta.getTipo() != null) {
				if (conta.getTipo().equals("C")) {
					radioTipoC.setChecked(Boolean.TRUE);
					radioTipoD.setChecked(Boolean.FALSE);
				} else {
					radioTipoC.setChecked(Boolean.FALSE);
					radioTipoD.setChecked(Boolean.TRUE);
				}
			}

			if (conta.getCancelamento() == null || !conta.getCancelamento()) {
				radioCancelamentoS.setChecked(Boolean.FALSE);
				radioCancelamentoN.setChecked(Boolean.TRUE);
			} else {
				radioCancelamentoS.setChecked(Boolean.TRUE);
				radioCancelamentoN.setChecked(Boolean.FALSE);
			}

			if (conta.getStatus() != null) {
				if (conta.getStatus()) {
					radioStatusA.setChecked(Boolean.TRUE);
					radioStatusI.setChecked(Boolean.FALSE);
				} else {
					radioStatusA.setChecked(Boolean.FALSE);
					radioStatusI.setChecked(Boolean.TRUE);
				}
			}
			
			if (conta.getOrigem() == null || conta.getOrigem().equals("IN")) {
				radioOrigemIN.setChecked(Boolean.TRUE);
				radioOrigemEE.setChecked(Boolean.FALSE);
				radioOrigemFP.setChecked(Boolean.FALSE);
			} else if (conta.getOrigem().equals("EE")) {
				radioOrigemIN.setChecked(Boolean.FALSE);
				radioOrigemEE.setChecked(Boolean.TRUE);
				radioOrigemFP.setChecked(Boolean.FALSE);
			} else if (conta.getOrigem().equals("FP")) {
				radioOrigemIN.setChecked(Boolean.FALSE);
				radioOrigemEE.setChecked(Boolean.FALSE);
				radioOrigemFP.setChecked(Boolean.TRUE);
			}
			
			if (conta.getReferencia() != null) {
				txtReferencia.setText(conta.getReferencia());
			}
		}

		txtNome.focus();
	}

	public void onClick$btnSalvar(Event ev) throws InterruptedException {
		txtCodigoContabil.getValue();
		txtNome.getValue();

		Integer codigoContabil = null;
		
		try {
			codigoContabil = txtCodigoContabil.getValueDecimal().intValue();
		} catch (Exception e) {
			Messagebox.show(
					Labels.getLabel("editarContaMDController.MSG.CodContabil.Invalido"),
					Labels.getLabel("editarContaMDController.window.title"),
					Messagebox.OK, Messagebox.ERROR);
			
			return;
		}
		
		try {
			conta.setCodigoContabil(codigoContabil);
			conta.setNome(txtNome.getText());

			if (radioTipoC.isChecked()) {
				conta.setTipo("C");
			} else if (radioTipoD.isChecked()) {
				conta.setTipo("D");
			}

			conta.setCancelamento(radioCancelamentoS.isChecked());
			conta.setStatus(radioStatusA.isChecked());
			
			if (radioOrigemIN.isChecked()) {
				conta.setOrigem("IN");
				conta.setReferencia(null);
			} else {
				if (radioOrigemEE.isChecked()) {
					conta.setOrigem("EE");
				} else if (radioOrigemFP.isChecked()) {
					conta.setOrigem("FP");
				}

				conta.setReferencia(txtReferencia.getText());
			}
			
			conta.setActivo(Boolean.TRUE);
			conta.setFecmodif(Calendar.getInstance().getTime());
			conta.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());

			if (!contaMDService.existeConta(conta)) {
				Messagebox.show(
						Labels.getLabel("editarContaMDController.MSG.DescricaoDuplicada"),
						Labels.getLabel("editarContaMDController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
				return;
			}

			if (conta.getContaId() == null) {
				contaMDService.suscribir(conta);
				contaList.addItemNovo(conta);

				Messagebox.show(
						Labels.getLabel("editarContaMDController.MSG.suscribirOK"),
						Labels.getLabel("editarContaMDController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
				
			} else {
				contaMDService.actualizacion(conta);
				contaList.updateItem(conta);

				Messagebox.show(
						Labels.getLabel("editarContaMDController.MSG.actualizarOK"),
						Labels.getLabel("editarContaMDController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			}

			closeWindow();
			
		} catch (Exception ex) {
			log.error(ex.getMessage(), ex);
			
			Messagebox.show(
					Labels.getLabel("MSG.Error"),
					Labels.getLabel("editarContaMDController.window.title"),
					Messagebox.OK, Messagebox.ERROR);
		}
	}

	public ContaMD getConta() {
		return conta;
	}

	public void setConta(ContaMD conta) {
		this.conta = conta;
	}

	public MyListbox getContaList() {
		return contaList;
	}

	public void setContaList(MyListbox contaList) {
		this.contaList = contaList;
	}

	public MyTextboxSimple getTxtNome() {
		return txtNome;
	}

	public void setTxtNome(MyTextboxSimple txtNome) {
		this.txtNome = txtNome;
	}

	public MyTextboxDecimal getTxtCodigoContabil() {
		return txtCodigoContabil;
	}

	public void setTxtCodigoContabil(MyTextboxDecimal txtCodigoContabil) {
		this.txtCodigoContabil = txtCodigoContabil;
	}

	public Radio getRadioTipoC() {
		return radioTipoC;
	}

	public void setRadioTipoC(Radio radioTipoC) {
		this.radioTipoC = radioTipoC;
	}

	public Radio getRadioTipoD() {
		return radioTipoD;
	}

	public void setRadioTipoD(Radio radioTipoD) {
		this.radioTipoD = radioTipoD;
	}

	public Radio getRadioStatusA() {
		return radioStatusA;
	}

	public void setRadioStatusA(Radio radioStatusA) {
		this.radioStatusA = radioStatusA;
	}

	public Radio getRadioStatusI() {
		return radioStatusI;
	}

	public void setRadioStatusI(Radio radioStatusI) {
		this.radioStatusI = radioStatusI;
	}

	public Radio getRadioOrigemIN() {
		return radioOrigemIN;
	}

	public void setRadioOrigemIN(Radio radioOrigemIN) {
		this.radioOrigemIN = radioOrigemIN;
	}

	public Radio getRadioOrigemEE() {
		return radioOrigemEE;
	}

	public void setRadioOrigemEE(Radio radioOrigemEE) {
		this.radioOrigemEE = radioOrigemEE;
	}

	public Radio getRadioOrigemFP() {
		return radioOrigemFP;
	}

	public void setRadioOrigemFP(Radio radioOrigemFP) {
		this.radioOrigemFP = radioOrigemFP;
	}

	public MyTextbox getTxtReferencia() {
		return txtReferencia;
	}

	public void setTxtReferencia(MyTextbox txtReferencia) {
		this.txtReferencia = txtReferencia;
	}
}
